﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace Lerak
{
    public enum Direction
    {
        Up = 0,
        Right = 1,
        Down = 2,
        Left = 3
    }

    public enum Moves
    {
        One = 0,
        Two = 1,
        Three = 2,
        Back = 3,
        Right = 4,
        Left = 5
    }

    public class Game
    {
        public const int HOLE = -7;
        public const int NOTHING = -4;
        public const int GOAL = -5;
        public const int WALL = -6;
        public const int TRACK_UP = 0;
        public const int TRACK_RIGHT = -1;
        public const int TRACK_DOWN = -2;
        public const int TRACK_LEFT = -3;
        public static readonly Point[] MoveOffset = new Point[] { new Point(0, -1), new Point(1, 0), new Point(0, 1), new Point(-1, 0) };
        public long ID = 0;
        public int[,] Map = new int[21, 21];
        public Point LerakPos = new Point();
        public Direction LerakDir = Direction.Up;
        public byte[] Possible = new byte[6];
        private byte[] DefaultPossible = new byte[6];
        private Point DefaultLerakPos = new Point();
        private Direction DefaultLerakDir = Direction.Up;
        public string ServerMessage = null;

        public static char DirToChar( Direction d )
        {
            switch ( d )
            {
                case Direction.Up:
                    {
                        return '▲';
                    }
                case Direction.Down:
                    {
                        return '▼';
                    }
                case Direction.Left:
                    {
                        return '◄';
                    }
                case Direction.Right:
                    {
                        return '►';
                    }
                default:
                    {
                        return ' ';
                    }
            }
        }

        public Game()
        {
            
        }

        public bool CanMove(int steps)
        {
            if (steps > 0)
            {
                while (steps != 0)
                {
                    steps--;
                    Point newPos = new Point(LerakPos.X, LerakPos.Y);
                    newPos.Offset(MoveOffset[(int)LerakDir]);
                    if ((newPos.X <= 20) && (newPos.Y <= 20) && (newPos.Y > 1) && (newPos.X > 1))
                    {
                        if (Map[newPos.X, newPos.Y] == Game.HOLE)
                        {
                            ResetLerak();
                            return false;
                        }
                        else if (Map[newPos.X, newPos.Y] != WALL)
                        {
                            //měníme pozici
                            LerakPos = newPos;
                        }
                    }
                    else
                    {
                        //vypadli jsme z hrací plochy
                        return false;
                    }
                }
                //finálová pozice je ještě posunuta pásem
                return MoveTrack();
            }
            else
            {
                //jdeme dozadu
                Point newPos = new Point(LerakPos.X, LerakPos.Y);
                Point p = MoveOffset[(int)LerakDir];
                p.X *= -1;
                p.Y *= -1;
                newPos.Offset(p);
                if ((newPos.X <= 20) && (newPos.Y <= 20) && (newPos.Y > 1) && (newPos.X > 1))
                {

                    if (Map[newPos.X, newPos.Y] == HOLE)
                    {
                        //padáme do díry
                        return false;
                    }
                    else if (Map[newPos.X, newPos.Y] != WALL)
                    {
                        //měníme pozici
                        LerakPos = newPos;
                        return MoveTrack();
                    }
                    else
                    {

                        return MoveTrack();
                    }
                }
                else
                {
                    //vypadli jsme z plochy
                    return false;
                }
            }
        }

        public bool MakeMove(int moveID)
        {
            Moves m = (Moves)Enum.ToObject(typeof(Moves), (byte)moveID);
            switch (m)
            {
                case Moves.One:
                    {
                        return CanMove(1);
                    }
                case Moves.Two:
                    {
                        return CanMove(2);
                    }
                case Moves.Three:
                    {
                        return CanMove(3);
                    }
                case Moves.Left:
                    {
                        int dir = ((int)LerakDir - 1) % 4;
                        if (dir < 0)
                        {
                            dir = 3;
                        }
                        LerakDir = (Direction)Enum.ToObject(typeof(Direction), dir);
                        return MoveTrack();                        
                    }
                case Moves.Right:
                    {
                        LerakDir = (Direction)Enum.ToObject(typeof(Direction), ((int)LerakDir + 1) % 4);
                        return MoveTrack();
                    }
                case Moves.Back:
                    {
                        return CanMove(-1);
                    }
                default:
                    {
                        return false;
                    }
            }            
        }

        public bool MoveTrack()
        {
            if ((Map[LerakPos.X, LerakPos.Y] <= 0) && (Map[LerakPos.X, LerakPos.Y] >= -3))
            {
                //stojíme na pásu
                Point newPos = LerakPos;
                newPos.Offset(MoveOffset[Math.Abs(Map[LerakPos.X, LerakPos.Y])]);
                if ((newPos.X <= 20) && (newPos.Y <= 20) && (newPos.Y > 1) && (newPos.X > 1))
                {
                    if (Map[newPos.X, newPos.Y] == HOLE)
                    {
                        //padáme do díry
                        return false;
                    }
                    else if (Map[newPos.X, newPos.Y] != WALL)
                    {
                        //měníme pozici
                        LerakPos = newPos;
                        return true;
                    }
                    else
                    {
                        return true;
                    }
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return true;
            }
        }

        public static int CharToMoveID(char c)
        {
            switch (Char.ToLower( c ))
            {
                case '1':
                    {
                        return 0;
                    }
                case '2':
                    {
                        return 1;
                    }
                case '3':
                    {
                        return 2;
                    }
                case 'z':
                    {
                        return 3;
                    }
                case 'l':
                    {
                        return 5;
                    }
                case 'r':
                    {
                        return 4;
                    }
                default:
                    {
                        return -1;
                    }
            }
        }

        /// <summary>
        /// Nastaví aktuální možné pohyby jako defaultní
        /// </summary>
        public void SetDefaultPossible()
        {
            for (int i = 0; i < 6; i++)
            {
                DefaultPossible[i] = Possible[i];
            }
        }

        public void SetDefaultLerak()
        {
            DefaultLerakPos.X = LerakPos.X;
            DefaultLerakPos.Y = LerakPos.Y;
            DefaultLerakDir = LerakDir;
        }

        /// <summary>
        /// Přepíše aktuální  možné pohyby defaultními
        /// </summary>
        public void ResetPossible()
        {
            for (int i = 0; i < 6; i++)
            {
                Possible[i] = DefaultPossible[i];
            }
        }

        public void ResetLerak()
        {
            LerakPos.X = DefaultLerakPos.X;
            LerakPos.Y = DefaultLerakPos.Y;
            LerakDir = DefaultLerakDir;
        }
    }
}
